#include <string.h>
#include <stdio.h>
#include <math.h>
#include "monomial.h"
#include "filepolynomutils.h"

#define USAGE "Ejemplo de uso : EvalTutte (-x | -y | -xy)  (valorX | valorY | valorX valorY) infile [outfile]\n"

#define HELPTEXT \
"Evalúa los polinomios de Tutte recibidos por fichero de entrada, reduciendo a una variable en caso de que únicamente se especifique el valor de una de las variables por entrada\n\
        \n\
        -x  : Para indicar el valor de la variable X.\n\
        -y  : Especifica el valor de la variable Y.\n\
        -xy  : Con este parámetro se pueden introducir los valores de X e Y.\n\
        \n\
        \n"


int main(int argc, char *argv[])
{
    monomial res;
    long double res1=0;
    FILE * f;
    FILE * f2;
    double x;
    double y;
    bool hasX=false;
    bool hasY=false;
    char * cadena;


    if (argc>1){
        if(strcmp(argv[1],"-xy")==0){
            hasX=true;
            hasY=true;
            if(argc==6){

                f=fopen(argv[4],"r");              
                if (f==NULL){
                    printf("\nNo se puede abrir el archivo %s\n",argv[4]);
                    return 1;
                }
                f2=fopen(argv[5],"w");
                if (f2==NULL){
                    printf("\nNo se puede crear el archivo %s\n",argv[5]);
                    return 1;
                }
                x=stringToDouble(argv[2]);
                y=stringToDouble(argv[3]);
                printf("\nEvaluando polinomios de %s en %s, para X = %g, Y = %g\n",argv[4],argv[5],x,y);
            }
            else if(argc==5){
                f=fopen(argv[4],"r");
                if (f==NULL){
                    printf("\nNo se puede abrir el archivo %s\n",argv[4]);
                    return 1;
                }
                f2=stdout;
                x=stringToDouble(argv[2]);
                y=stringToDouble(argv[3]);
                printf("\nEvaluando polinomios de %s , para X = %g, Y = %g\n",argv[4],x,y);

            }

            else{
                    printf("Parámetros de entrada incorrectos\n");
                    printf("\n");
                    printf("%s",USAGE);
                    return 1;
                }
        }
        else if(strcmp(argv[1],"-y")==0){
            hasY=true;
            if(argc==5){

                f=fopen(argv[3],"r");
                if (f==NULL){
                    printf("\nNo se puede abrir el archivo %s\n",argv[3]);
                    return 1;
                }
                f2=fopen(argv[4],"w");
                y=stringToDouble(argv[2]);
                printf("\nEvaluando polinomios de %s en %s, para Y = %g\n",argv[3],argv[4],y);
            }
            else if(argc==4){

                 f=fopen(argv[3],"r");
                 if (f==NULL){
                    printf("\nNo se puede abrir el archivo %s\n",argv[3]);
                    return 1;
                }
                 f2=stdout;
                 y=stringToDouble(argv[2]);
                 printf("\nEvaluando polinomios de %s , para Y = %g\n",argv[3],y);

            }
                else{
                    printf("Parámetros de entrada incorrectos\n");
                    printf("\n");
                    printf("%s",USAGE);
                    return 1;
                }
        }
        else if(strcmp(argv[1],"-x")==0){
            hasX=true;
            if(argc==5){

                f=fopen(argv[3],"r");
                if (f==NULL){
                    printf("\nNo se puede abrir el archivo %s\n",argv[4]);
                    return 1;
                }
                f2=fopen(argv[4],"w");
                x=stringToDouble(argv[2]);
                printf("\nEvaluando polinomios de %s en %s, para X = %g\n",argv[3],argv[4],x);
            }
            else if(argc==4){

                f=fopen(argv[3],"r");
                if (f==NULL){
                    printf("\nNo se puede abrir el archivo %s\n",argv[3]);
                    return 1;
                }
                f2=stdout;
                x=stringToDouble(argv[2]);
                printf("\nEvaluando polinomios de %s , para X = %g\n",argv[3],x);

            }
                else{
                    printf("Parámetros de entrada incorrectos\n");
                    printf("\n");
                    printf("%s",USAGE);
                    return 1;
                }
        }
        else if((strcmp(argv[1],"-help")==0 || strcmp(argv[1],"--help")==0)&& argc==2){
            printf("\n");
            printf("%s",USAGE);
            printf("\n");
            printf("%s",HELPTEXT);
            return 1;
        }
        else{
            printf("Parámetros de entrada incorrectos\n");
            printf("\n");
            printf("%s",USAGE);
            return 1;
        }
    }
    else{
        printf("Parámetros de entrada incorrectos\n");
        printf("\n");
        printf("%s",USAGE);
        return 1;
    }

    if(hasX && hasY){
        cadena = getGraphLine(f);
        fprintf(f2,"%s",cadena);
        fprintf(f2,"\n");
        res1 = evaluatePolynom(f,x,y);
        while(res1!=NULL){
            fprintf(f2,"%LG\n",res1);
            res1=0;
            cadena = getGraphLine(f);
            if(cadena!=NULL)
                fprintf(f2,"%s",cadena);
            fprintf(f2,"\n");
            res1 = evaluatePolynom(f,x,y);
        }
    }
    else if(!hasX && hasY){
        cadena = getGraphLine(f);
        fprintf(f2,"%s",cadena);
        fprintf(f2,"\n");
        res = evaluatePolynomY(f,y);
        while(res.getFlag()!=NULL){
            res.printMonomialXInFile(f2);
            res.cleanMonomial();
            cadena = getGraphLine(f);
            if(cadena!=NULL)
                fprintf(f2,"%s",cadena);
            fprintf(f2,"\n");
            res = evaluatePolynomY(f,y);
        }

    }
    else if(hasX && !hasY){
        cadena = getGraphLine(f);
        fprintf(f2,"%s",cadena);
        fprintf(f2,"\n");
        res = evaluatePolynomX(f,x);
        while(res.getFlag()!=NULL){
            res.printMonomialYInFile(f2);
            res.cleanMonomial();
            cadena = getGraphLine(f);
            if(cadena!=NULL)
                fprintf(f2,"%s",cadena);
            fprintf(f2,"\n");
            res = evaluatePolynomX(f,x);
        }
    }
    else{
        printf("\nError: No se capturaron parámetros de entrada válidos\n");
        return 1;
    }
    fclose(f2);
    fclose(f);
    return 1;
}

